############################################################################
##
## Copyright (c) 2000-2001 BalaBit IT Ltd, Budapest, Hungary
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
## $Id: IPChains.py,v 1.1.2.1 2001/01/29 17:23:22 bazsi Exp $
##
############################################################################
import Zorp
from PF import PacketFilter
from Domain import Domain

CORE_IPCHAINS = 'core.ipchains'

AF_INET = 2

# Values for "fw_flg" field
IP_FW_F_PRN     = 0x0001  # Print packet if it matches 
IP_FW_F_TCPSYN  = 0x0002  # For tcp packets-check SYN only 
IP_FW_F_FRAG    = 0x0004  # Set if rule is a fragment rule 
IP_FW_F_MARKABS = 0x0008  # Set the mark to fw_mark, not add. 
IP_FW_F_WILDIF  = 0x0010  # Need only match start of interface name. 
IP_FW_F_NETLINK = 0x0020  # Redirect to netlink: 2.1.x only 
IP_FW_F_MASK    = 0x003F  # All possible flag bits mask   

# Values for "fw_invflg" field
IP_FW_INV_SRCIP = 0x0001  # Invert the sense of fw_src. 
IP_FW_INV_DSTIP = 0x0002  # Invert the sense of fw_dst. 
IP_FW_INV_PROTO = 0x0004  # Invert the sense of fw_proto. 
IP_FW_INV_SRCPT = 0x0008  # Invert the sense of source ports. 
IP_FW_INV_DSTPT = 0x0010  # Invert the sense of destination ports. 
IP_FW_INV_VIA   = 0x0020  # Invert the sense of fw_vianame. 
IP_FW_INV_SYN   = 0x0040  # Invert the sense of IP_FW_F_TCPSYN. 
IP_FW_INV_FRAG  = 0x0080  # Invert the sense of IP_FW_F_FRAG. 


def calcIpMask(dom):
	if isinstance(dom, Domain):
		if dom.family != AF_INET:
			raise ValueError, 'IPChains supports only AF_INET address domains'
		return dom.ip, dom.mask
	else:
		return dom.ip, 0xffffffff


class IPChains(PacketFilter):
	# rule: (target, (src, smask), (dst, dmask), mark, proto, flags, inv_flags, (sportmin, sportmax), (dportmin, dportmax), redirport, iface, tosand, tosxor)
	
	def __init__(self):
		self.rules = []
		self.ipchains = None
		self.ipchains = Zorp.IPChains()
	
	def __del__(self):
		if self.ipchains:
			for rule in self.rules[:]:
				self.delRule(rule)

	def addRule(self, rule):
		apply(self.ipchains.addRule, rule)
		Zorp.log(CORE_IPCHAINS, 7, 'adding rule: %s' % (rule,))
		self.rules.append(rule)
	
	def delRule(self, rule):
		Zorp.log(CORE_IPCHAINS, 7, 'deleting rule: %s' % (rule,))
		apply(self.ipchains.delRule, rule)
		self.rules.remove(rule)

	def addREDIRECT(self, chain, sdom, sports, ddom, dports, redirport, iface="", mark=0, proto=0, flags=0, inv_flags=0, tosand=0xff, tosxor=0x00):
		rule = (chain, ('REDIRECT', calcIpMask(sdom), calcIpMask(ddom), 0, proto, flags, inv_flags, sports, dports, redirport, iface, tosand, tosxor))
		self.addRule(rule)
		return rule
	
	def addACCEPT(self, chain, sdom, sports, ddom, dports, iface="", mark=0, proto=0, flags=0, inv_flags=0, redirport=0, tosand=0xff, tosxor=0x00):
		rule = (chain, ('ACCEPT', calcIpMask(sdom), calcIpMask(ddom), 0, proto, flags, inv_flags, sports, dports, redirport, iface, tosand, tosxor))
		self.addRule(rule)
		return rule

	def addDENY(self, chain, sdom, sports, ddom, dports, iface="", mark=0, proto=0, flags=0, inv_flags=0, redirport=0, tosand=0xff, tosxor=0x00):
		rule = (chain, ('DENY', calcIpMask(sdom), calcIpMask(ddom), 0, proto, flags, inv_flags, sports, dports, redirport, iface, tosand, tosxor))
		self.addRule(rule)
		return rule

	def addREJECT(self, chain, sdom, sports, ddom, dports, iface="", mark=0, proto=0, flags=0, inv_flags=0, redirport=0, tosand=0xff, tosxor=0x00):
		rule = (chain, ('REJECT', calcIpMask(sdom), calcIpMask(ddom), 0, proto, flags, inv_flags, sports, dports, redirport, iface, tosand, tosxor))
		self.addRule(rule)
		return rule

	def addRETURN(self, chain, sdom, sports, ddom, dports, iface="", mark=0, proto=0, flags=0, inv_flags=0, redirport=0, tosand=0xff, tosxor=0x00):
		rule = (chain, ('RETURN', calcIpMask(sdom), calcIpMask(ddom), 0, proto, flags, inv_flags, sports, dports, redirport, iface, tosand, tosxor))
		self.addRule(rule)
		return rule
	
